#!/usr/bin/perl

# usage: iso2022jp-map.pl < cp232.txt > iso2022jp-map.h

while (<STDIN>) {
    if (!(/^#/ or /^$/)) {
	  my ($sjis, $ucode, $cmt) = split(/\t/);

	  $jis = oct $sjis;
	  $ucode = oct $ucode;

	  if (($jis) > 0xff) {
	      $s1 = oct substr($sjis,0,4);
	      $s2 = oct "0x".substr($sjis,4,2);
	      
	      $s1 -= ($s1 <= 0x9f) ? 0x71 : 0xb1;
	      $s1 = $s1 * 2 + 1;
	      $s2-- if ($s2 > 0x7f);
	      if ($s2 >= 0x9e) {
		  $s2 -= 0x7d;
		  $s1++;
	      } else {
		  $s2 -= 0x1f;
	      }
	      
	      $jis = $s1 * 256 + $s2;
	  }
	  if ($ucode != 0) {
	      $j2u{$jis} = $ucode;
	      $u2j{$ucode} = $jis;
	  }
     }
}



#open (CHDR, ">iso2022jp-map.h");
#    select (CHDR);

    print <<_HEADER_;
#ifndef _ISO2022JP_MAP_
#define _ISO2022JP_MAP_
/*
  Unicode <=> ISO-2022-JP Character conversion map.
  Based on Microsoft's Unicode map available at:
     http://www.unicode.org/Public/MAPPINGS/VENDORS/MICSFT/WINDOWS/CP932.TXT

  Automatically created with perl script by Norihisa Washitake.
  (C) 2002, Norihisa Washitake <nori@washitake.com>

*/

_HEADER_

# first, j2u.
    print "/* map: iso-2022-jp to Unicode */\n";
    for ($hb=0x21; $hb<0x7f; $hb++) {
	$items = 0;
	for ($lb=0x21; $lb<0x7f; $lb++) {
	    $items++ if ($j2u{$hb*256 + $lb} > 0);
	}
	if ($items > 0) {
	    $items = 0;
	    printf "static const int jis2uni_tbl_%02x[] = {", $hb;
	    for ($lb = 0x21; $lb < 0x7f; $lb++) {
		$real = $hb*256 + $lb;
		print ", " if ($items > 0);
		print "\n  " if ($items % 8 == 0);
                $j2u{$real} = 0x003f if ($j2u{$real} == 0);
		printf("0x%04X", $j2u{$real});
		$items++;
	    }
	    print "\n};\n";
	    $j2uout{$hb} = 1;
	}
    }

    print "const int * jis2uni_tbls[] = {\n";
    for ($hb=0x21; $hb<0x7f; $hb++) {
	print (($hb > 0x21) ? ",\n  " : "  ");
	if ($j2uout{$hb} > 0) {
	    printf "jis2uni_tbl_%02x", $hb;
	}else {
	    print "NULL";
	}
    }
    print "\n};\n";

    print "\n\n";

#next, u2j.
    print "/* map : Unicode to iso-2022-jp */\n";
    for ($hb=0x00; $hb<=0xff; $hb++) {
	$items = 0;
	for ($lb=0x0; $lb<=0xff; $lb++) {
	    $items++ if ($u2j{$hb*256 + $lb} > 0);
	}
	if ($items > 0) {
	    $items = 0;
	    printf "static const int uni2jis_tbl_%02x[] = {", $hb;
	    for ($lb = 0x00; $lb <= 0xff; $lb++) {
		$real = $hb * 256 + $lb;
		print ", " if ($items > 0);
		print "\n  " if ($items % 8 == 0);
                $u2j{$real} = 0x003f if ($u2j{$real} == 0);
		printf("0x%04X", $u2j{$real});
		$items++;
	    }
	    print "\n};\n";
	    $u2jout{$hb} = 1;
	}
    }

    print "const int * uni2jis_tbls[] = {\n";
    for ($hb = 0x00; $hb <= 0xff; $hb++) {
	print (($hb > 0x00) ? ",\n  " : "  ");
	if ($u2jout{$hb} > 0) {
	    printf "uni2jis_tbl_%02x", $hb;
	}else {
	    print "NULL";
	}
    }
    print "\n};\n";

print "#endif /* _ISO2022JP_MAP_ */\n";
    select(STDOUT);
#    close(CHDR);

__END__
